--
-- Transmission
-- Specialization for obtaining and updating the Transmission of a vehicle
--
-- @author  Templaer
-- @date  03/12/09

Transmission = {};

function Transmission.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Motorized, specializations);
end;

function Transmission:load(xmlFile)
	self.modifyRpm = SpecializationUtil.callSpecializationsFunction("modifyRpm");	
	self.motorManagement = SpecializationUtil.callSpecializationsFunction("motorManagement");	
	
	self.backupMaxRpm = self.motor.maxRpm[3];
	self.backupMinRpm = self.motor.minRpm;
end;

function Transmission:delete()
end;

function Transmission:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Transmission:keyEvent(unicode, sym, modifier, isDown)
end;

function Transmission:update(dt)
	if self.isEntered then
		if g_currentMission.allowSteerableMoving and not self.playMotorSound then
			self:modifyRpm();
			self:motorManagement();
		end;
	end;
end;

function Transmission:draw()
	if self.isEntered and self.isMotorStarted then
		local rpm = math.floor(((self.lastRoundPerMinute + 300)/30) + 0.5) * 30;
		self:addTransmissionStats("Current RPM:", string.format("%d RPM", math.min(rpm , self.motor.maxRpm[3]))); 
		self:addTransmissionStats(string.format("%s/%s:", InputBinding.getButtonKeyName(InputBinding.RAISERPM), InputBinding.getButtonKeyName(InputBinding.LOWERRPM)), string.format("%d RPM", self.motor.maxRpm[self.motor.speedLevel]));
		self:addTransmissionStats(string.format("%s/%s/%s:", InputBinding.getButtonKeyName(InputBinding.SPEED_LEVEL1), InputBinding.getButtonKeyName(InputBinding.SPEED_LEVEL2), InputBinding.getButtonKeyName(InputBinding.SPEED_LEVEL3)), string.format("Tempomat %d", self.motor.speedLevel));
	end;
end;

function Transmission:onLeave()
	self.motor.minRpm = self.backupMinRpm;
end;

function Transmission:stopMotor()
	self.motor.minRpm = self.backupMinRpm;
end;

-- Additional functions
function Transmission:modifyRpm()
	if self:getIsActiveForInput() then
		if InputBinding.isPressed(InputBinding.RAISERPM) then
			if self.motor.maxRpm[self.motor.speedLevel] < self.backupMaxRpm then
				self.motor.maxRpm[self.motor.speedLevel] = self.motor.maxRpm[self.motor.speedLevel] + 10;
			end;
		end;
		
		if InputBinding.isPressed(InputBinding.LOWERRPM) then
			if self.motor.maxRpm[self.motor.speedLevel] > 200 then
				self.motor.maxRpm[self.motor.speedLevel] = self.motor.maxRpm[self.motor.speedLevel] - 10;
			end;
		end;
	end;
end;

function Transmission:motorManagement()
	if self.motor.lastMotorRpm < (self.motor.maxRpm[self.motor.speedLevel] - 200) or self.motor.lastMotorRpm > self.motor.maxRpm[self.motor.speedLevel] then
		if self.lastRoundPerMinute < math.abs(self.acceleration) * self.backupMaxRpm then
			self.motor.minRpm = self.motor.minRpm - 15;
		else
			self.motor.minRpm = math.min(self.motor.minRpm + 15, self.backupMinRpm);
		end;
	else
		if self.lastRoundPerMinute < (self.backupMaxRpm * 0.6) then
			self.motor.minRpm = self.motor.minRpm - 15;
		elseif self.lastRoundPerMinute > (self.backupMaxRpm * 0.625) then
			self.motor.minRpm = self.motor.minRpm + 15;
		end;
	end;
end;